__title__ = "unrest"
__version__ = "0.7.2"

__summary__ = "A troubling rest api library for sqlalchemy models"
__uri__ = "https://github.com/Kozea/unrest"
__author__ = "Florian Mounier"
__email__ = "florian.mounier@kozea.fr"

__license__ = "GNU LGPL v3+"
__copyright__ = "Copyright 2017 %s" % __author__
